/*
 * IEEE 1284 Device ID tool
 *
 * Copyright (C) 1999  Tim Waugh <tim@cyberelk.demon.co.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 */

#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <getopt.h>
#include "parport.h"
#include "ppdev.h"

#define BUFSIZE 4000

enum
{
  Mfg = 1,
  Mdl = 2,
  Cmd = 4
};

static void version (void)
{
  printf ("deviceid 0.1\n");
  exit (0);
}

static void syntax (void)      
{
  printf ("Usage: deviceid [options] PORT\n\n"
	  "  --help               show this message\n"
	  "  --version            display version information and exit\n"
	  "  --mfg                show manufacturer\n"
	  "  --cmd                show command set\n"
	  "  --mdl                show model\n\n"
	  "If none of --mfg, --cmd, or --mdl are specified, output is the unprocessed\n"
	  "device ID of the peripheral (without the length field).\n");
  exit (0);
}

static int get_deviceid (int fd, int fields)
{
  char buffer[BUFSIZE + 1];
  ssize_t len = 0;
  FILE *f;

  if (read (fd, buffer, 2) < 2)
    {
      fprintf (stderr, "deviceid: no length field? (!)\n");
      return 1;
    }

  f = fdopen (fd, "r");
  if (!f)
    {
      perror ("deviceid: fdopen");
      return 1;
    }

  for (;;)
    {
      char ch = buffer[len++] = fgetc (f);

      if (ch == ';' || ch == EOF)
	{
	  /* End of a field. */
	  int field;

	  if (ch == EOF)
	    len--;

	  buffer[len] = '\0';

	  if (!fields)
	    fputs (buffer, stdout);
	  else
	    {
	      if (ch != EOF)
		buffer[--len] = '\0';
	      buffer[len++] = ' ';
	      buffer[len] = '\0';

	      if (fields & Mfg)
		{
		  if (!strncasecmp (buffer, "manufacturer:", 13))
		    fputs (buffer + 13, stdout);
		  else if (!strncasecmp (buffer, "mfg:", 4))
		    fputs (buffer + 4, stdout);
		}

	      if (fields & Mdl)
		{
		  if (!strncasecmp (buffer, "model:", 6))
		    fputs (buffer + 6, stdout);
		  else if (!strncasecmp (buffer, "mdl:", 4))
		    fputs (buffer + 4, stdout);
		}

	      if (fields & Cmd)
		{
		  if (!strncasecmp (buffer, "command set:", 12))
		    fputs (buffer + 12, stdout);
		  else if (!strncasecmp (buffer, "cmd:", 4))
		    fputs (buffer + 4, stdout);
		}
	    }
	  

	  len = 0;
	}

      if (ch == EOF)
	break;
    }

  putchar ('\n');
  return 0;
}

int main (int argc, char *argv[])
{
  int fd;
  int ret;
  int fields = 0;
  int compat = IEEE1284_MODE_COMPAT;
  int mode = IEEE1284_MODE_NIBBLE;

  for (;;)
    {
      static struct option long_opts[] =
      {
	{ "help", 0, 0, 'h' },
	{ "version", 0, 0, 'v' },
	{ "mfg", 0, 0, Mfg },
	{ "cmd", 0, 0, Cmd },
	{ "mdl", 0, 0, Mdl },
	{ 0, 0, 0, 0 }
      };
      int c = getopt_long (argc, argv, "s", long_opts, NULL);

      if (c == -1)
	break;

      switch (c)
	{
	case 'h':
	  syntax ();

	case 'v':
	  version ();

	case Mfg:
	case Cmd:
	case Mdl:
	  fields |= c;
	  break;
	}
    }

  if (argc - optind != 1)
    syntax ();

  fd = open (argv[optind], O_RDONLY | O_NOCTTY);

  if (fd == -1)
    {
      perror (argv[optind]);
      return fd;
    }

  if (ioctl (fd, PPSETMODE, &mode))
    {
      fprintf (stderr, "deviceid: cannot set transfer mode\n");
      close (fd);
      return 1;
    }

  if (ioctl (fd, PPCLAIM))
    {
      fprintf (stderr, "deviceid: cannot claim port\n");
      close (fd);
      return 1;
    }

  if (ioctl (fd, PPNEGOT, &compat))
    {
      perror ("PPNEGOT");
      close (fd);
      return 1;
    }

  mode |= IEEE1284_DEVICEID;
  if (ioctl (fd, PPNEGOT, &mode))
    {
      fprintf (stderr, "deviceid: device not IEEE 1284 compliant\n");
      close (fd);
      return 1;
    }

  ret = get_deviceid (fd, fields);

  ioctl (fd, PPNEGOT, &compat);
  ioctl (fd, PPRELEASE);
  close (fd);

  return ret;
}
